<?php

/**
 * @author          Tassos Marinos <info@tassos.gr>
 * @link            http://www.tassos.gr
 * @copyright       Copyright © 2021 Tassos Marinos All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 */

namespace NRFramework\Widgets;

defined('_JEXEC') or die;

use Joomla\Registry\Registry;
use NRFramework\Helpers\Widgets\GalleryManager as GalleryManagerHelper;

/**
 *  Gallery Manager
 */
class GalleryManager extends Widget
{
	/**
	 * Widget default options
	 *
	 * @var array
	 */
	protected $widget_options = [
		// The input name
		'name' => '',

		// The folder where the images will be uploaded to.
		'upload_folder' => 'media/gallery_manager',

		/**
		 * Max file size in MB.
		 * 
		 * Defults to 0 (no limit).
		 */
		'max_file_size' => 0,

		// How many files we can upload
		'limit_files' => 0,

		// Allowed upload file types
		'allowed_file_types' => '.jpg, .jpeg, .png',

		// When we upload existing images (via Media Manager), should the image be copied to the defined upload folder?
		'copy_existing_images' => false,

		/**
		 * Original Image
		 */
		// Should the original uploaded image be resized?
		'original_image_resize' => false,

		// Original Image Resize Quality
		'original_image_resize_quality' => 80,

		// Main image width
		'original_image_resize_width' => 1920,

		/**
		 * Thumbnails
		 */
		// Thumbnails width
		'thumb_width' => 300,

		// Thumbnails height
		'thumb_height' => 300,

		// Resize method (crop, stretch, fit)
		'resize_method' => 'crop',

		// Resize quality
		'resize_quality' => 80
	];

	public function __construct($options = [])
	{
		parent::__construct($options);

		// Set gallery items
		$this->options['gallery_items'] = is_array($this->options['value']) ? $this->options['value'] : [];

		// Set widget settings, this is passed to the HTML Element in the data-widget-settings attribute
		$widget_settings = $this->options;
		unset($widget_settings['value']);
		unset($widget_settings['gallery_items']);
		$this->options['widget_settings'] = $widget_settings;

		// Set css class for readonly state
		if ($this->options['readonly'])
		{
			$this->options['css_class'] .= ' readonly';
		}

		// Load translation strings
        \JText::script('NR_GALLERY_MANAGER_CONFIRM_DELETE_ALL_SELECTED');
        \JText::script('NR_GALLERY_MANAGER_CONFIRM_DELETE_ALL');
        \JText::script('NR_GALLERY_MANAGER_CONFIRM_DELETE');
        \JText::script('NR_GALLERY_MANAGER_FILE_MISSING');
	}

	/**
	 * The upload task called by the AJAX hanler
	 *
	 * @return  void
	 */
	protected function ajax_upload()
	{
		$input = \JFactory::getApplication()->input;

		// Make sure we have the widget settings
		if (!$widget_settings = $input->get('widget_settings', [], 'raw'))
		{
			$this->exitWithMessage('NR_GALLERY_MANAGER_WIDGET_SETTINGS_MISSING');
		}

		// Parse Widget Settings
		if (!$widget_settings = json_decode($widget_settings, true))
		{
			$this->exitWithMessage('NR_GALLERY_MANAGER_WIDGET_SETTINGS_INVALID');
		}

		// Make sure we have a valid file passed
		if (!$file = $input->files->get('file', null, null))
		{
			$this->exitWithMessage('NR_GALLERY_MANAGER_ERROR_INVALID_FILE');
		}

		$widget_settings = new Registry($widget_settings);

		// get the media uploader file data, values are passed when we upload a file using the Media Uploader
		$media_uploader_file_data = [
			'is_media_uploader_file' => $input->get('media_uploader', false) == '1',
			'media_uploader_filename' => $input->getString('media_uploader_filename', ''),
			// Whether the image was copied to the `upload_folder` when added via the media uploader
			'copied_from_media_uploader' => $input->get('media_uploader', false) == '1' && $widget_settings->get('copy_existing_images', false)
		];

		// In case we allow multiple uploads the file parameter is a 2 levels array.
		$first_property = array_pop($file);
		if (is_array($first_property))
		{
			$file = $first_property;
		}

		$uploadSettings = [
			'upload_folder' => $widget_settings->get('upload_folder'),
			'copy_existing_images' => ($widget_settings->get('copy_existing_images', false)),
			'allow_unsafe' => false,
			'allowed_types' => $widget_settings->get('allowed_file_types', '')
		];

		// resize image settings
		$resizeSettings = [
			'style' => $widget_settings->get('style', 'masonry'),
			'thumb_width' => $widget_settings->get('thumb_width', 300),
			'thumb_height' => $widget_settings->get('thumb_height', 300),
			'resize_method' => $widget_settings->get('resize_method', 'crop'),
			'resize_quality' => $widget_settings->get('resize_quality', 80),
			'original_image_resize' => ($widget_settings->get('original_image_resize', false)),
			'original_image_resize_width' => $widget_settings->get('original_image_resize_width', 1920),
			'original_image_resize_quality' => $widget_settings->get('original_image_resize_quality', 80)
		];

		// Upload the file and resize the image if needed
		if (!$uploaded_filenames = GalleryManagerHelper::upload($file, $uploadSettings, $media_uploader_file_data, $resizeSettings))
		{
			$this->exitWithMessage('NR_GALLERY_MANAGER_ERROR_CANNOT_UPLOAD_FILE');
		}

		echo json_encode([
			'filename' => $uploaded_filenames['filename'],
			'thumbnail' => $uploaded_filenames['thumbnail'],
			'is_media_uploader_file' => $media_uploader_file_data['is_media_uploader_file'],
			'copied_from_media_uploader' => $media_uploader_file_data['copied_from_media_uploader']
		]);
	}

	/**
	 * The delete task called by the AJAX hanlder
	 *
	 * @return void
	 */
	protected function ajax_delete()
	{
		$input = \JFactory::getApplication()->input;

		// Make sure we have a valid file passed
		if (!$filename = $input->getString('filename'))
		{
			$this->exitWithMessage('NR_GALLERY_MANAGER_ERROR_INVALID_FILE');
		}

		// Make sure we have the widget settings
		if (!$widget_settings = $input->get('widget_settings', [], 'raw'))
		{
			$this->exitWithMessage('NR_GALLERY_MANAGER_WIDGET_SETTINGS_MISSING');
		}

		// Parse Widget Settings
		if (!$widget_settings = json_decode($widget_settings, true))
		{
			$this->exitWithMessage('NR_GALLERY_MANAGER_WIDGET_SETTINGS_INVALID');
		}

		$widget_settings = new Registry($widget_settings);

		// Delete the uploaded file
		$deleted = GalleryManagerHelper::deleteFile(
			$filename,
			$input->getString('thumbnail'),
			$widget_settings->get('upload_folder'),
			[
				'copied_from_media_uploader' => $input->get('copied_from_media_uploader') === 'true',
				'is_media_uploader_file' => $input->get('is_media_uploader_file') === 'true'
			]
		);
		
		echo json_encode(['success' => $deleted]);
	}

	/**
	 * Exits the page with given message.
	 * 
	 * @param   string  $translation_string
	 * 
	 * @return  void
	 */
	private function exitWithMessage($translation_string)
	{
		http_response_code('500');
		die(\JText::_($translation_string));
	}
}